'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.editFields = editFields;
exports.changeFocus = changeFocus;
exports.initForm = initForm;
exports.changeFieldValue = changeFieldValue;

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var actionTypes = _config2.default.actionTypes;
function editFields(id, fields) {
  return {
    type: actionTypes.editFields,
    payload: { id: id, fields: fields }
  };
}

function changeFocus(id, element, type) {
  var name = element.name;

  return {
    type: actionTypes.changeFocus,
    payload: {
      id: id,
      fields: _defineProperty({}, name, { active: type === 'focus' ? true : false })
    }
  };
}

function initForm(_ref) {
  var id = _ref.id;
  var fields = _ref.fields;
  var onChange = _ref.onChange;
  var validate = _ref.validate;

  return function (dispatch, getState) {
    dispatch({
      type: actionTypes.initForm,
      payload: { id: id, fields: fields }
    });
    applyAdditionalChanges(id, onChange, dispatch, getActualFields(getState(), id));
    performValidation(id, validate, dispatch, getState);
  };
}

function changeFieldValue(_ref2) {
  var id = _ref2.id;
  var element = _ref2.element;
  var onChange = _ref2.onChange;
  var validate = _ref2.validate;

  return function (dispatch, getState) {
    var fields = elementToField(element);
    dispatch(editFields(id, fields));
    applyAdditionalChanges(id, onChange, dispatch, fields);
    performValidation(id, validate, dispatch, getState);
  };
}

function applyAdditionalChanges(id, onChange, dispatch, fields) {
  var changes = onChange && onChange(fields);
  if (changes && Object.keys(changes).length > 0) {
    dispatch({
      type: actionTypes.additionalChanges,
      payload: { id: id, fields: changes }
    });
  }
}

function performValidation(id, validate, dispatch, getState) {
  var newFields = getActualFields(getState(), id);
  var errors = validate && validate(newFields);
  if (validate) {
    dispatch({
      type: actionTypes.setErrors,
      payload: { id: id, fields: errors }
    });
  }
}

function getActualFields(state, id) {
  return _config2.default.selector(state).getIn([id, 'fields']).toJS();
}

function elementToField(element) {
  var name = element.name;
  var type = element.type;
  var value = element.value;

  var field = {};
  switch (type) {
    case 'checkbox':
      if (value === 'true' || value === 'false') {
        field.value = element.checked;
      } else {
        field.fields = _defineProperty({}, value, { checked: element.checked });
      }
      break;

    case 'select':
      field.value = optionsToValue(element.options)[0];
      break;

    case 'select-multiple':
      field.value = optionsToValue(element.options);
      break;

    case undefined:
    default:
      field.value = value;
  }
  return _defineProperty({}, name, field);
}

function optionsToValue(options) {
  return [].concat(_toConsumableArray(options)).filter(function (o) {
    return o.selected;
  }).map(function (o) {
    return o.value;
  });
}