'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = function () {
  var state = arguments.length <= 0 || arguments[0] === undefined ? initialState : arguments[0];
  var action = arguments[1];

  switch (action.type) {
    case actionTypes.initForm:
      return initForm(state, action);

    case actionTypes.changeFocus:
    case actionTypes.additionalChanges:
      return additionalChanges(state, action);

    case actionTypes.editFields:
      return editFields(state, action);

    case actionTypes.setErrors:
      return setErrors(state, action);

    case actionTypes.destroyForm:
      return destroyForm(state, action);

    default:
      return state;
  }
};

exports.replaceArrays = replaceArrays;

var _isPlainObject = require('lodash/isPlainObject');

var _isPlainObject2 = _interopRequireDefault(_isPlainObject);

var _isArray = require('lodash/isArray');

var _isArray2 = _interopRequireDefault(_isArray);

var _mergeWith = require('lodash/mergeWith');

var _mergeWith2 = _interopRequireDefault(_mergeWith);

var _immutable = require('immutable');

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var actionTypes = _config2.default.actionTypes;

var initialState = (0, _immutable.Map)();

function initForm(state, action) {
  var _action$payload = action.payload;
  var id = _action$payload.id;
  var fields = _action$payload.fields;

  return state.setIn([id, 'fields'], initFields((0, _immutable.fromJS)(fields))).setIn([id, 'initialized'], true);
}

function destroyForm(state, action) {
  var id = action.payload.id;

  return state.delete(id);
}

function initFields(fields) {
  var result = {};
  fields.forEach(function (value, key) {
    var schema = value.toJS && value.toJS() || value;
    var field = {};

    if ((0, _isPlainObject2.default)(schema)) {
      field = schema;
    } else {
      field.value = schema;
      if (typeof schema === 'boolean') {
        field.checked = schema;
      }
    }

    if (!field.hasOwnProperty('initialValue')) {
      field.initialValue = field.value || '';
    }
    if (!field.hasOwnProperty('value')) {
      field.value = field.initialValue;
    }

    if (field.fields) {
      if (!(0, _isPlainObject2.default)(field.fields)) {
        (function () {
          var obj = {};
          field.fields.forEach(function (f) {
            return obj[f] = {};
          });
          field.fields = obj;
        })();
      }

      Object.keys(field.fields).forEach(function (f) {
        field.fields[f] = _extends({}, field.fields[f], {
          checked: isChecked(f, field.value)
        });
      });
    }
    result[key] = field;
  });
  result[_config2.default.nonFieldErrorsKey] = { errors: null };
  return (0, _immutable.Map)(result);
}

function isChecked(fieldName, value) {
  if (!value) return false;
  return value.indexOf(fieldName) !== -1;
}

function editFields(state, action) {
  return iterateThroughFields(state, action, setValue);
}

function iterateThroughFields(state, action, fn) {
  var _action$payload2 = action.payload;
  var id = _action$payload2.id;
  var fields = _action$payload2.fields;

  Object.keys(fields).forEach(function (key) {
    var field = key;
    var props = fields[key];
    state = state.updateIn([id, 'fields', field], function (f) {
      var newField = (0, _mergeWith2.default)({}, f, props, replaceArrays);
      if (fn) {
        newField = fn(newField, props);
      }
      return newField;
    });
  });
  return state;
}

function setValue(newField, props) {
  // If value is provided with props, then update all `checked` props if they are exist.
  // If value is NOT provided, but `checked` IS, then update `value` prop.
  if (props.hasOwnProperty('value')) {
    if (newField.fields) {
      Object.keys(newField.fields).forEach(function (k) {
        newField.fields[k].checked = newField.value.indexOf(k) !== -1;
      });
    } else {
      if (newField.hasOwnProperty('checked')) {
        newField.checked = newField.value;
      }
    }
  } else {
    if (newField.fields) {
      newField.value = Object.keys(newField.fields).filter(function (i) {
        return newField.fields[i].checked;
      });
    } else {
      if (props.hasOwnProperty('checked')) {
        newField.value = props.checked;
      }
    }
  }
  return newField;
}

function additionalChanges(state, action) {
  return iterateThroughFields(state, action);
}

function setErrors(state, action) {
  var _action$payload3 = action.payload;
  var id = _action$payload3.id;
  var fields = _action$payload3.fields;

  var adjustedFields = {};
  var immutableFields = (0, _immutable.fromJS)(fields);
  state.getIn([id, 'fields']).forEach(function (v, k) {
    adjustErrorField(adjustedFields, immutableFields, v, k);
  });
  var newAction = (0, _immutable.fromJS)(action);
  newAction = newAction.setIn(['payload', 'fields'], adjustedFields);
  state = iterateThroughFields(state, newAction.toJS());
  var hasErrors = state.getIn([id, 'fields']).some(function (f) {
    return f.errors;
  });
  state = state.setIn([id, 'errors'], hasErrors);
  return state;
}

function adjustErrorField(adjustedFields, actionFields, stateField, key) {
  var errors = null;
  var value = actionFields.get(key);
  if (value) {
    value = value.toJS && value.toJS() || value;
    if (typeof value === 'string') {
      errors = [value];
    } else {
      errors = value;
    }
  }
  if (value || stateField.errors) {
    adjustedFields[key] = { errors: errors };
  }
}

function replaceArrays(objValue, srcValue) {
  if ((0, _isArray2.default)(objValue)) {
    return srcValue;
  }
}