'use strict';

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _redux = require('redux');

var _reduxThunk = require('redux-thunk');

var _reduxThunk2 = _interopRequireDefault(_reduxThunk);

var _sinon = require('sinon');

var _chai = require('chai');

var _chai2 = _interopRequireDefault(_chai);

var _enzyme = require('enzyme');

var _chaiEnzyme = require('chai-enzyme');

var _chaiEnzyme2 = _interopRequireDefault(_chaiEnzyme);

var _sinonChai = require('sinon-chai');

var _sinonChai2 = _interopRequireDefault(_sinonChai);

var _Form = require('../containers/Form');

var _Form2 = _interopRequireDefault(_Form);

var _reducers = require('../reducers');

var _reducers2 = _interopRequireDefault(_reducers);

var _config = require('../config');

var _config2 = _interopRequireDefault(_config);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var actionTypes = _config2.default.actionTypes;

_chai2.default.use((0, _chaiEnzyme2.default)());
_chai2.default.use(_sinonChai2.default);

var fields = {
  username: 'john',
  sex: { fields: ['male', 'female'], value: 'male' },
  agree: false,
  hands: { fields: ['left', 'right'], value: [] },
  fruits: { fields: ['tomato', 'potato'], value: 'tomato' },
  pets: { fields: ['rat', 'python'], value: ['rat'] }
};

function onChange() {
  return {
    fruits: {
      fields: { potato: { disabled: true } }
    }
  };
}

function validate() {
  return {
    username: 'Too many johnes'
  };
}

describe('Form container', function () {
  var spiedReducer = void 0;
  var createForm = void 0;

  beforeEach(function () {
    spiedReducer = (0, _sinon.spy)((0, _redux.combineReducers)({ form: _reducers2.default }));
    var store = (0, _redux.createStore)(spiedReducer, {}, (0, _redux.applyMiddleware)(_reduxThunk2.default));
    createForm = function createForm(onChange, validate) {
      return (0, _enzyme.mount)(_react2.default.createElement(
        _Form2.default,
        { store: store, id: 'form1', fields: fields, onChange: onChange, validate: validate },
        _react2.default.createElement('input', { id: 'username', name: 'username' }),
        _react2.default.createElement('input', { id: 'male', name: 'sex', value: 'male', type: 'radio' }),
        _react2.default.createElement('input', { id: 'female', name: 'sex', value: 'female', type: 'radio' }),
        _react2.default.createElement('input', { id: 'agree', name: 'agree', type: 'checkbox' }),
        _react2.default.createElement('input', { id: 'left', name: 'hands', value: 'left', type: 'checkbox' }),
        _react2.default.createElement('input', { id: 'right', name: 'hands', value: 'right', type: 'checkbox' }),
        _react2.default.createElement(
          'select',
          { id: 'fruits', name: 'fruits' },
          _react2.default.createElement(
            'option',
            { id: 'tomato', value: 'tomato' },
            'Tomato'
          ),
          _react2.default.createElement(
            'option',
            { value: 'potato' },
            'Potato'
          )
        ),
        _react2.default.createElement(
          'select',
          { id: 'pets', name: 'pets', multiple: true },
          _react2.default.createElement(
            'option',
            { value: 'rat' },
            'Rat'
          ),
          _react2.default.createElement(
            'option',
            { value: 'python' },
            'Python'
          )
        ),
        _react2.default.createElement('input', { id: 'excess', name: 'excess' }),
        _react2.default.createElement(
          'button',
          null,
          'Submit'
        )
      ));
    };
  });

  it('requires id and fields props', function () {
    var dispatch = (0, _sinon.spy)();
    (0, _chai.expect)(function () {
      (0, _enzyme.mount)(_react2.default.createElement(_Form.UnconnectedForm, { dispatch: dispatch, id: 'form1' }));
    }).to.throw(Error);
    (0, _chai.expect)(function () {
      (0, _enzyme.mount)(_react2.default.createElement(_Form.UnconnectedForm, { dispatch: dispatch, fields: {} }));
    }).to.throw(Error);
    (0, _chai.expect)(function () {
      (0, _enzyme.mount)(_react2.default.createElement(_Form.UnconnectedForm, { dispatch: dispatch, id: 'form1', fields: {} }));
    }).to.not.throw(Error);
  });

  describe('when mounted', function () {
    function testInitFormAction() {
      (0, _chai.expect)(spiedReducer).to.have.been.calledWith(_sinon.match.any, {
        type: actionTypes.initForm,
        payload: {
          id: 'form1',
          fields: fields
        }
      });
    }

    describe('without onChange and validate', function () {
      it('dispatches INIT_FORM action', function () {
        createForm();
        testInitFormAction();
        (0, _chai.expect)(spiedReducer.callCount).to.eql(2);
      });
    });

    describe('with onChange', function () {
      it('dispatches INIT_FORM, ADDITIONAL_CHANGES actions', function () {
        createForm(onChange);
        testInitFormAction();
        (0, _chai.expect)(spiedReducer).to.have.been.calledWith(_sinon.match.any, {
          type: actionTypes.additionalChanges,
          payload: {
            id: 'form1',
            fields: {
              fruits: {
                fields: { potato: { disabled: true } }
              }
            }
          }
        });
        (0, _chai.expect)(spiedReducer.callCount).to.eql(3);
      });
    });

    describe('with validate', function () {
      it('dispatches INIT_FORM, SET_ERRORS actions', function () {
        createForm(undefined, validate);
        testInitFormAction();
        (0, _chai.expect)(spiedReducer).to.have.been.calledWith(_sinon.match.any, {
          type: actionTypes.setErrors,
          payload: {
            id: 'form1',
            fields: {
              username: 'Too many johnes'
            }
          }
        });
        (0, _chai.expect)(spiedReducer.callCount).to.eql(3);
      });
    });
  });

  describe('when unmounted', function () {
    it('dispatches DESTROY_FORM action', function () {
      var form = createForm();
      form.unmount();
      (0, _chai.expect)(spiedReducer).to.have.been.calledWith(_sinon.match.any, {
        type: actionTypes.destroyForm,
        payload: { id: 'form1' }
      });
    });
  });

  describe('on render', function () {
    it('provides children which have prop `name` with corresponding field props', function () {
      var form = createForm(onChange);
      (0, _chai.expect)(form.find("#username")).to.have.prop('value', 'john');
      (0, _chai.expect)(form.find("#male")).to.have.prop('checked', true);
      (0, _chai.expect)(form.find("#female")).to.have.prop('checked', false);
      (0, _chai.expect)(form.find("#agree")).to.have.prop('value', false);
      (0, _chai.expect)(form.find("#tomato")).to.be.selected();
      (0, _chai.expect)(Object.keys(form.find("#excess").props()).length).to.eql(2);
      (0, _chai.expect)(Object.keys(form.find("button").props()).length).to.eql(1); // children 'Submit'
    });

    it('provides children which have prop `hoth-form` with props returned by calling `hoth-form` function');
  });

  describe('onFieldChange function', function () {
    it('calls `onChange` function if exists');
    it('calls `validate` function if exists');

    it('handles input changes', function () {
      var form = createForm();
      form.find('#username').simulate('change', { target: { name: 'username', value: 'mark' } });
      (0, _chai.expect)(spiedReducer).to.have.been.calledWith(_sinon.match.any, {
        type: actionTypes.editFields,
        payload: {
          id: 'form1',
          fields: { username: { value: 'mark' } }
        }
      });
    });

    it('handles radio changes', function () {
      var form = createForm();
      form.find('#female').simulate('change', { target: { name: 'sex', value: 'female', checked: true } });
      (0, _chai.expect)(spiedReducer).to.have.been.calledWith(_sinon.match.any, {
        type: actionTypes.editFields,
        payload: {
          id: 'form1',
          fields: { sex: { value: 'female' } }
        }
      });
    });

    it('handles single checkbox changes', function () {
      var form = createForm();
      form.find('#agree').simulate('change', { target: {
          name: 'agree',
          type: 'checkbox',
          value: 'false',
          checked: true
        } });
      (0, _chai.expect)(spiedReducer).to.have.been.calledWith(_sinon.match.any, {
        type: actionTypes.editFields,
        payload: {
          id: 'form1',
          fields: { agree: { value: true } }
        }
      });
    });

    it('handles multiple checkbox changes', function () {
      var form = createForm();
      form.find('#left').simulate('change', { target: {
          name: 'hands',
          type: 'checkbox',
          value: 'left',
          checked: true
        } });
      (0, _chai.expect)(spiedReducer).to.have.been.calledWith(_sinon.match.any, {
        type: actionTypes.editFields,
        payload: {
          id: 'form1',
          fields: {
            hands: { fields: { left: { checked: true } } }
          }
        }
      });
    });

    it('handles select changes', function () {
      var form = createForm();
      form.find('#fruits').simulate('change', { target: {
          name: 'fruits',
          type: 'select',
          options: [{ selected: false, value: 'tomato' }, { selected: true, value: 'potato' }]
        } });
      (0, _chai.expect)(spiedReducer).to.have.been.calledWith(_sinon.match.any, {
        type: actionTypes.editFields,
        payload: {
          id: 'form1',
          fields: { fruits: { value: 'potato' } }
        }
      });
    });

    it('handles multiple select changes', function () {
      var form = createForm();
      form.find('#pets').simulate('change', { target: {
          name: 'pets',
          type: 'select-multiple',
          options: [{ selected: true, value: 'rat' }, { selected: true, value: 'python' }]
        } });
      (0, _chai.expect)(spiedReducer).to.have.been.calledWith(_sinon.match.any, {
        type: actionTypes.editFields,
        payload: {
          id: 'form1',
          fields: {
            pets: { value: ['rat', 'python'] }
          }
        }
      });
    });
  });
});